import pymongo
import trafilatura
import trafilatura.feeds
import trafilatura.sitemaps
import trafilatura.spider
import sys
import courlan


def calculate_checksums(text):
    """
    @return fingerprints of a paragraphs in text. Paragraphs are separated by a blank line
    """
    checksums = []
    sizes = []
    hval = 0
    hsz = 0
    sz = 0
    for c in text:
        cv = ord(c)
        sz += 1
        if cv > 64:
            hval += (hval << 3) + cv
            zv = hval >> 31
            hval &= 0x7fffffff
            hval += zv
            hsz += 1
        if c == "\n" and hsz > 0:
            if hsz > 100:
                checksums.append(hval)
                sizes.append(sz)
            sz = 0
            hsz = 0
    if hsz > 100:
        checksums.append(hval)
        sizes.append(sz)
    return checksums, sizes




def filter_links(links):
    out = set()
    for link in links:
        r = courlan.check_url(link)
        if r is None:
            continue
        link,ldomain = r
        # domain rules
        if not ldomain.endswith("sk"):
            continue
        if not courlan.is_crawlable(link):
            continue
        out.add(link)
    return out


def generic_visit(domain):
    known_links = set(get_visited_links(domain))
    visit_links = []
    visit_links = trafilatura.find_feed_urls(domain)
    if visit_links is None:
        visit_links = trafilatura.sitemap_search(domain)
    if visit_links is None:
        visit_links = trafilatura.focused_crawler(dommain,known_links=known_links)


def get_new_links(domain):
    known_links = []
    # get seed links
    # get new links
    out_links = trafilatura.spider.focused_crawler(domain,known_links=known_links)
    filtered_links = filter_links(out_links)
    print(res)
    return filtered_links

def fetch_links(link_batch):
    htmls  = []
    print(link_batch)
    print("zzzzzzzzzz")
    for link in link_batch:
        print(link)
        rr = trafilatura.fetch_url(link,decode=True)
        htmls.append(rr)
    return htmls


def extract_links(link_batch,htmls):
    out = []
    for link,html in zip(link_batch,htmls):
        doc = None
        assert link is not None
        if html is not None:
            doc = trafilatura.bare_extraction(html,url=link,include_links=True,with_metadata=True,include_formatting=True,target_language="sk")
        out.append((link,html,doc))
    return out

def index_pages(db,extracted_pages,domain):
    extracted_links = set()
    linkcol = db["links"]
    htmlcol = db["html"]
    contentcol = db["content"]
    for link,html,doc in extracted_pages:
        state = "good"
        if html is None:
            state = "html_error"
        elif doc is None:
            state = "content_error"
        if html is not None:
            htmlcol.insert_one({"url":link,"html":html})
        if doc is not None:
            print(doc)
            checksums,sizes = calculate_checksums(doc["text"])
            doc["paragraph_checksums"] = checksums
            doc["paragraph_sizes"] = sizes
            if "links" in doc:
                extracted_links.union(doc["links"])
                del doc["links"]
            contentcol.insert_one(doc)
        linkcol.replace_one({"url":link},{"url":link,"status":state},upsert=True)
        filtered_links = filter_links(extracted_links)
        for link in filtered_links:
            linkcol.insert_one({"url":link,"status":"backlink"},upsert=True)


def simple_visit(start_link):
    known_links = []
    visit_links,known_links = trafilatura.spider.focused_crawler(start_link,known_links=known_links)
    htmls = fetch_links(visit_links)
    extracted_links = extract_links(visit_links,htmls)
    myclient = pymongo.MongoClient("mongodb://root:example@localhost:27017/")

    db = myclient["mydatabase"]
    index_pages(db,extracted_links,start_link)
    #visit_links = trafilatura.feeds.find_feed_urls(domain)
    #visit_links = trafilatura.sitemaps.sitemap_search(domain)
    #print(visit_links)
    #for link in visit_links:
    #    content = trafilatura.fetch_url(link,decode=True)
    #    document = trafilatura.bare_extraction(content)
    #    print(content)

simple_visit(sys.argv[1])
